@@ -1,18 +1,14 @@
 import curses
 from dataclasses import dataclass, field
-from typing import (
-    Generic,
-    Callable,
-    List,
-    Optional,
-    Dict,
-    Union,
-    Tuple,
-    TypeVar,
-    Sequence,
-)
+from typing import Any, List, Optional, Sequence, Tuple, TypeVar, Union, Generic
 
-__all__ = ["Picker", "pick"]
+__all__ = ["Picker", "pick", "Option"]
+
+
+@dataclass
+class Option:
+    label: str
+    value: Any
 
 
 KEYS_ENTER = (curses.KEY_ENTER, ord("\n"), ord("\r"))
@@ -23,25 +19,19 @@ KEYS_SELECT = (curses.KEY_RIGHT, ord(" "))
 SYMBOL_CIRCLE_FILLED = "◉"
 SYMBOL_CIRCLE_EMPTY = "◯"
 
-CUSTOM_HANDLER_RETURN_T = TypeVar("CUSTOM_HANDLER_RETURN_T")
-KEY_T = int
-OPTIONS_MAP_VALUE_T = TypeVar("OPTIONS_MAP_VALUE_T")
-PICK_RETURN_T = Tuple[OPTIONS_MAP_VALUE_T, int]
+OPTION_T = TypeVar("OPTION_T", str, Option)
+PICK_RETURN_T = Tuple[OPTION_T, int]
 
 
 @dataclass
-class Picker(Generic[CUSTOM_HANDLER_RETURN_T, OPTIONS_MAP_VALUE_T]):
-    options: Sequence[OPTIONS_MAP_VALUE_T]
+class Picker(Generic[OPTION_T]):
+    options: Sequence[OPTION_T]
     title: Optional[str] = None
     indicator: str = "*"
     default_index: int = 0
     multiselect: bool = False
     min_selection_count: int = 0
-    options_map_func: Callable[[OPTIONS_MAP_VALUE_T], Optional[str]] = str
     selected_indexes: List[int] = field(init=False, default_factory=list)
-    custom_handlers: Dict[KEY_T, Callable[["Picker"], CUSTOM_HANDLER_RETURN_T]] = field(
-        init=False, default_factory=dict
-    )
     index: int = field(init=False, default=0)
     scroll_top: int = field(init=False, default=0)
 
@@ -57,16 +47,8 @@ class Picker(Generic[CUSTOM_HANDLER_RETURN_T, OPTIONS_MAP_VALUE_T]):
                 "min_selection_count is bigger than the available options, you will not be able to make any selection"
             )
 
-        if not callable(self.options_map_func):
-            raise ValueError("options_map_func must be a callable function")
-
         self.index = self.default_index
 
-    def register_custom_handler(
-        self, key: KEY_T, func: Callable[["Picker"], CUSTOM_HANDLER_RETURN_T]
-    ) -> None:
-        self.custom_handlers[key] = func
-
     def move_up(self) -> None:
         self.index -= 1
         if self.index < 0:
@@ -117,7 +99,7 @@ class Picker(Generic[CUSTOM_HANDLER_RETURN_T, OPTIONS_MAP_VALUE_T]):
                 )
                 prefix = f"{prefix} {symbol} "
 
-            option_as_str = self.options_map_func(option)
+            option_as_str = option.label if isinstance(option, Option) else option
             lines.append(f"{prefix} {option_as_str}")
 
         return lines
@@ -153,9 +135,7 @@ class Picker(Generic[CUSTOM_HANDLER_RETURN_T, OPTIONS_MAP_VALUE_T]):
 
         screen.refresh()
 
-    def run_loop(
-        self, screen
-    ) -> Union[List[PICK_RETURN_T], PICK_RETURN_T, CUSTOM_HANDLER_RETURN_T]:
+    def run_loop(self, screen) -> Union[List[PICK_RETURN_T], PICK_RETURN_T]:
         while True:
             self.draw(screen)
             c = screen.getch()
@@ -172,10 +152,6 @@ class Picker(Generic[CUSTOM_HANDLER_RETURN_T, OPTIONS_MAP_VALUE_T]):
                 return self.get_selected()
             elif c in KEYS_SELECT and self.multiselect:
                 self.mark_index()
-            elif c in self.custom_handlers:
-                ret = self.custom_handlers[c](self)
-                if ret:
-                    return ret
 
     def config_curses(self) -> None:
         try:
@@ -187,27 +163,22 @@ class Picker(Generic[CUSTOM_HANDLER_RETURN_T, OPTIONS_MAP_VALUE_T]):
             # Curses failed to initialize color support, eg. when TERM=vt100
             curses.initscr()
 
-    def _start(
-        self, screen
-    ) -> Union[List[PICK_RETURN_T], PICK_RETURN_T, CUSTOM_HANDLER_RETURN_T]:
+    def _start(self, screen):
         self.config_curses()
         return self.run_loop(screen)
 
-    def start(
-        self,
-    ) -> Union[List[PICK_RETURN_T], PICK_RETURN_T, CUSTOM_HANDLER_RETURN_T]:
+    def start(self):
         return curses.wrapper(self._start)
 
 
 def pick(
-    options: Sequence[OPTIONS_MAP_VALUE_T],
+    options: Sequence[OPTION_T],
     title: Optional[str] = None,
     indicator: str = "*",
     default_index: int = 0,
     multiselect: bool = False,
     min_selection_count: int = 0,
-    options_map_func: Callable[[OPTIONS_MAP_VALUE_T], Optional[str]] = str,
-) -> Union[List[PICK_RETURN_T], PICK_RETURN_T]:
+):
     picker: Picker = Picker(
         options,
         title,
@@ -215,6 +186,5 @@ def pick(
         default_index,
         multiselect,
         min_selection_count,
-        options_map_func,
     )
     return picker.start()
